/*
	╔═══════════════════════════════╗
	║ mds.c:                        ║
	║    Monitor de disponibilidad. ║
	║                               ║
	║ Autores: Jose Luis Araya      ║
	║          Cristobal Ganter     ║
	╚═══════════════════════════════╝
*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <unistd.h>
#include <signal.h>
#include <assert.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>

/*__CONSTANTES Y TIPOS__*/
const long int interval = 60;
enum result {false, true};
typedef enum result result;
typedef char* str;


/*__GLOBALES__*/
str dns, url, cpu, mem, smtp, mail, auth;


/*__PROTOTIPOS__*/
void
check (int signum);

int
dns_check (char ip[20]);

result
usage (char max_per [], char document [], char col []);

result
web_check (char * URL);

void
mail_deamon(result a, result b, result c, result d, char servidor [], char email [], char codificacion64 []);


/*__MAIN__*/
int
main (int argc, char** argv)
{
	printf("\n");

	if (argc != 8)
		{
			printf ("Uso: %s <IP_DNS> <URL> <CPU%%> <M%%> <servidor_smtp> <email> <autenticacion_codificada>\n\n", argv[0]);
			return;
		}

	dns  = argv[1];
	url  = argv[2];
	cpu  = argv[3];
	mem  = argv[4];
	smtp = argv[5];
	mail = argv[6];
	auth = argv[7];

	void * signal_result;
	signal_result = signal (SIGALRM, check);
	if (signal_result == SIG_ERR)
		{
			fprintf (stderr, "Error al setear el handler!");
			return;
		}

	struct itimerval new;
	new.it_interval.tv_usec = 0;
	new.it_interval.tv_sec = interval;
	new.it_value.tv_usec = 0;
	new.it_value.tv_sec = 1;

	int setitimer_result;
	setitimer_result = setitimer (ITIMER_REAL, &new, NULL);
	if (setitimer_result == -1)
		{
			fprintf (stderr, "Error al setear el timer!");
			return;
		}

	while (1)
		pause ();
}

void
check (int signum)
{
	FILE *erase;

	result dnsr = dns_check (dns);
	result webr = web_check (url);
	result cpur = usage (cpu, "cputest.txt", "$9");
	result memr = usage (mem, "memtest.txt", "$10");

	if (dnsr || webr || cpur || memr)
		mail_deamon(dnsr, webr, cpur, memr, smtp, mail, auth);
	else
		printf ("No se han encontrado fallas. :)\n");

        if (( erase = popen("rm  memtest.txt cputest.txt httpcheck.txt dnstemp.txt","w")) == NULL) // PIPE QU$
        {
                        perror("popen");
                        exit(1);
        }
        pclose(erase);

}

int
dns_check(char ip[20])
{
        FILE *pipe_dns, *pipe_verification;
        int resultado;

        char command[50];
        snprintf(command, sizeof command, "nslookup - -timeout=2 %s > dnstemp.txt", ip);

        if (( pipe_dns = popen(command, "w")) == NULL)
        {
                perror("popen");
                exit(1);
        }

        fprintf(pipe_dns,"www.google.com");
        fflush(pipe_dns);
        pclose(pipe_dns);

        if (( pipe_verification = popen("./dnsbash.sh", "r")) == NULL)
        {
                perror("popen");
                exit(1);
        }

        fscanf(pipe_verification,"%d",&resultado);

        return resultado;
}

result
usage(char max_per [], char document [], char col [])
{
        FILE *pipe_write, *archivo;	// descriptores de archivos que permite escribir a la pipa y leer de documeto creado en ambient shell
        int i = 0;				//contador de while
        float max = 0;			//máximo valor a tomar indicado en la entrada del programa.Es transformado a float desde la entrada char
        float percentage = 0, temp = 0;	//valores que se modificaran a partir de la lectura

        sscanf(max_per, "%f", &max);	//max_percentage entra como char, se transforma a float

        char bash_command[100], var_document[20];
        snprintf(bash_command, sizeof bash_command, "top -b -n 1 | tail -n +8 | awk '{print %s}' > %s", col, document);

        if (( pipe_write = popen(bash_command,"w")) == NULL)
        {			//Se abre pipa para poder escribir la columna del top correspondiente al CPU
                perror("popen");//en archivo de texto. Esto se hace debido a que durante la ejecuccion del
                exit(1);	//proceso es probable que el iterador cambie porcentajes y de esta forma la
        }			//salida no es exacta. 
        pclose(pipe_write);	// Se cierra pipa ya que lo que resta se obtiene del documento txt.

        snprintf(var_document, sizeof var_document, "%s", document);	//Se genera el nombre del texto a abrir
	archivo = fopen(var_document,"r");				//Se abre doc txt
        while(!feof(archivo))
        {
                fscanf(archivo, "%f", &temp);	//Se lee el número y se va sumando a la variable percentage
                percentage = percentage + temp;
        }


        fclose(archivo);

        if(max > percentage)		//Condición respecto a que se hace cuando se supera el máximo.
                return false;
        else
                return true;

}

result
web_check (char * URL)
{
	char dir[50] = "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0";
	str rec;
	char frec[] = "/";
	
	rec = strchr (URL, '/');
	if (rec == NULL)
		{
			strcpy (dir, URL);
			rec = frec;
		}
	else
		strncpy (dir, URL, rec-URL);
	
	char command[300] = "telnet ";
	strcat (command, dir);
	strcat (command, " 80 | grep HTTP > httpcheck.txt");

	FILE * cmdipipe = popen (command, "w");
	if (cmdipipe == NULL)
		{
			fprintf (stderr, "Error al abrir el pipe!\n");
			exit (EXIT_FAILURE);
		}

	sleep (2);
	fprintf (cmdipipe, "HEAD %s HTTP/1.1\nHost: %s\n\n", rec, dir);
	fflush (cmdipipe);
	sleep (2);
	pclose (cmdipipe);

	FILE * cmdout = fopen ("httpcheck.txt", "r");
		assert (cmdout != NULL);
		char trash[10];
		int status;
		int matches = fscanf (cmdout, "%s %d ", trash, &status);
	fclose (cmdout);

	if (matches != 2)
		{
			fprintf (stderr, "Ha habido un error al leer el archivo\n");
			exit (EXIT_FAILURE);
		}

	if (status >= 200 && status < 400)
		return false;
	else
		return true;
}

void
mail_deamon(result a, result b, result c, result d, char servidor [], char email [], char codificacion64 [])
{
	pid_t pid;
	int i, status;
	FILE *pipa_ssl;

	char falla[50] = "";
	if (a) strcat(falla,"DNS ");
	if (b) strcat(falla,"WEB ");
	if (c) strcat(falla,"CPU ");
	if (d) strcat(falla,"MEM ");
	if (d) strcat(falla,"\b");
	printf ("Se han encontrado las siguientes fallas: %s\nEnviando e-mail de notificacion...\n", falla);

	if ((pipa_ssl = popen("openssl s_client -starttls smtp -crlf -connect smtp.gmail.com:587 > mailtemp.txt","w")) == NULL)
		{
			perror("popen");
        	exit(1);
        }

	sleep(2);

	char cuerpo[] = "Los siguientes sistemas han fallado:";

	fprintf(pipa_ssl, "AUTH PLAIN %s\n", codificacion64);
	fflush(pipa_ssl);
	sleep (1);

	fprintf(pipa_ssl, "mail from: <%s>\n", email);
	fflush(pipa_ssl);
	sleep (1);

	fprintf(pipa_ssl, "rcpt to: <%s>\n", email);
	fflush(pipa_ssl);
	sleep (1);

	fprintf(pipa_ssl, "DATA\n");
	fflush(pipa_ssl);
	sleep (1);

	fprintf(pipa_ssl, "From: \"Monitor\" <%s>\nTo: \"Admin\" <%s>\nSubject: Falla en %s.\n\n%s %s\n.\n", email, email, falla, cuerpo, falla);
	fflush(pipa_ssl);
	sleep (1);

	fprintf(pipa_ssl, "quit\n");
	fflush(pipa_ssl);
	sleep(1);

	pclose (pipa_ssl);
	
	remove ("mailtemp.txt");
}










